(define-module (librefinder utils)
  #:use-module (oop goops)
  #:export (make-sanitizer
	    make-maybe-sanitizer
	    make-list-predicate

	    strings?
	    exact-positive-integer?
	    single-predicate?
	    single-predicates?
	    pairs?

	    sanitize-string
	    sanitize-?string
	    sanitize-strings
	    sanitize-exact-positive-integer
	    sanitize-?number
	    sanitize-single-predicate
	    sanitize-single-predicates
	    sanitize-record-type
	    sanitize-pairs
	    sanitize-?pairs))

(define-syntax make-sanitizer
  (lambda (x)
    "Make a sanitizer for type TYPE"
    (syntax-case x ()
      ((_ type)
       #`(define (#,(datum->syntax
		     x
		     (string->symbol
		      (string-append "sanitize-"
				     (symbol->string
				      (syntax->datum #'type))))) value)
	   #,(string-append "Sanitize TYP to be "
			    (symbol->string (syntax->datum #'type))
			    " type.")
	   (if (#,(datum->syntax
		   x
		   (string->symbol
		    (string-append (symbol->string (syntax->datum #'type))
				   "?"))) value)
	       value
	       (throw 'bad! value)))))))

(define-syntax make-maybe-sanitizer
  (lambda (x)
    "Make a sanitizer for type TYPE"
    (syntax-case x ()
      ((_ type)
       #`(define (#,(datum->syntax
		     x
		     (string->symbol
		      (string-append "sanitize-?"
				     (symbol->string
				      (syntax->datum #'type))))) value)
	   #,(string-append "Sanitize TYP to be "
			    (symbol->string (syntax->datum #'type))
			    " type or `#f'.")
	   (if (or (not value)
		   (#,(datum->syntax
		       x
		       (string->symbol
			(string-append (symbol->string (syntax->datum #'type))
				       "?"))) value))
	       value
	       (throw 'bad! value)))))))

(define-syntax make-list-predicate
  (lambda (x)
    "Make a predicate for a list compount only by TYPE."
    (syntax-case x ()
      ((_ type)
       #`(begin
	   (use-modules (srfi srfi-1))
	   (define (#,(datum->syntax
		       x
		       (string->symbol
			(string-append (symbol->string
					(syntax->datum #'type))
				       "s?"))) lst)
	     #,(string-append
		"Return `#t' if LST is a list and every element is of type"
		(symbol->string (syntax->datum #'type))
		", else return `#f'.")
	     (and (list? lst)
		  (fold (lambda (elem rest)
			  (and rest
			       (#,(datum->syntax
				   x
				   (string->symbol
				    (string-append (symbol->string
						    (syntax->datum #'type))
						   "?")))
				elem)))
			#t lst))))))))

(make-sanitizer string)
(make-maybe-sanitizer string)

(make-list-predicate string)
(make-sanitizer strings)

(define (exact-positive-integer? value)
  (and (number? value)
       (exact-integer? value)
       (positive? value)))

(make-sanitizer exact-positive-integer)

(define (single-predicate? value)
  (and (procedure? value)))

(make-sanitizer single-predicate)
(make-list-predicate single-predicate)
(make-sanitizer single-predicates)

(make-maybe-sanitizer number)

(make-sanitizer record-type)

(make-list-predicate pair)
(make-maybe-sanitizer pairs)
